@@ -174,8 +174,7 @@ class FeatureTournament:
                                                                                  multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
                                                                                  multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
                                                                                  log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
-                                                                                 verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
-                                                                                 **self.kwargs
+                                                                                 verbose=False if self.kwargs.get('verbose') is None else self.kwargs.get('verbose')
                                                                                  )
         elif self.evolutionary_algorithm == 'si':
             _feature_tournament_ai_learning: SwarmIntelligence = SwarmIntelligence(mode='model',
@@ -204,8 +203,7 @@ class FeatureTournament:
                                                                                    multi_threading=False if self.kwargs.get('multi_threading') is None else self.kwargs.get('multi_threading'),
                                                                                    multi_processing=False if self.kwargs.get('multi_processing') is None else self.kwargs.get('multi_processing'),
                                                                                    log=False if self.kwargs.get('log') is None else self.kwargs.get('log'),
-                                                                                   verbose=0 if self.kwargs.get('verbose') is None else self.kwargs.get('verbose'),
-                                                                                   **self.kwargs
+                                                                                   verbose=False if self.kwargs.get('verbose') is None else self.kwargs.get('verbose')
                                                                                    )
         else:
             raise FeatureTournamentException('Reinforced evolutionary algorithm ({}) not supported'.format(self.evolutionary_algorithm))
