@@ -5,12 +5,17 @@ Text clustering algorithms
 """
 
 import numpy as np
+import math
+import pandas as pd
+import pyLDAvis
 
+from easyexplore.data_import_export import DataExporter
+from fast_gsdmm import GSDMM
 from gensim import corpora
 from gensim.models import LdaModel, LsiModel
 from sklearn.decomposition import NMF
 from sklearn.feature_extraction.text import TfidfVectorizer
-from typing import Dict, List
+from typing import Dict, List, Tuple
 
 
 def _rank_clusters(model, text_data: List[str], vocab: corpora.Dictionary) -> tuple:
@@ -42,15 +47,16 @@ class GibbsSamplingDirichletMultinomialModeling:
     Class for building Gibbs Sampling Dirichlet Multinomial Modeling model
     """
     def __init__(self,
-                 vocab_size: int,
+                 vocab: List[str],
                  n_clusters: int = 8,
                  n_iterations: int = 30,
                  alpha: float = 0.1,
-                 beta: float = 0.1
+                 beta: float = 0.1,
+                 fast: bool = True,
                  ):
         """
-        :param vocab_size: int
-            Size of the vocabulary
+        :param vocab: List[str]
+            Vocabulary
 
         :param n_clusters: int
             Number of pre-defined clusters
@@ -63,50 +69,78 @@ class GibbsSamplingDirichletMultinomialModeling:
 
         :param beta: float
             Affinity of a cluster element with same characteristics as other elements
+
+        :param fast: bool
+            Use C++ or Python implementation of the fit & document_scoring methods
         """
+        self.fast: bool = fast
         self.alpha: float = alpha
         self.beta: float = beta
         self.n_clusters: int = n_clusters
         self.n_iterations: int = n_iterations if n_iterations > 5 else 30
         self.n_documents: int = 0
-        self.vocab_size: int = vocab_size
+        self.vocab: List[str] = list(set(vocab))
+        self.vocab_size: int = len(self.vocab)
         self.probability_vector: List[float] = []
         self.cluster_word_count: List[int] = [0 for _ in range(0, self.n_clusters, 1)]
         self.cluster_document_count: List[int] = [0 for _ in range(0, self.n_clusters, 1)]
         self.cluster_word_distribution: List[dict] = [{} for _ in range(0, self.n_clusters, 1)]
+        if self.fast:
+            self.model: GSDMM = GSDMM(self.vocab,
+                                      self.vocab_size,
+                                      self.n_clusters,
+                                      self.n_iterations,
+                                      self.alpha,
+                                      self.beta
+                                      )
+        else:
+            self.model = None
 
-    def _document_scoring(self, documents: List[str]) -> List[float]:
+    def _document_scoring(self, document: List[str]) -> List[float]:
         """
         Score a document
 
-        :param documents: List[str]:
-            The document token stream
+        :param document: List[str]:
+            Tokenized document
 
         :return: list[float]:
             Probability vector where each component represents the probability of the document appearing in a particular cluster
         """
-        _p = [0 for _ in range(0, self.n_clusters, 1)]
-        #  We break the formula into the following pieces
-        #  p = n1 * n2 / (d1 * d2) = np.exp(ln1 - ld1 + ln2 - ld2)
-        #  lN1 = np.log(m_z[z] + alpha)
-        #  lN2 = np.log(D - 1 + K*alpha)
-        #  lN2 = np.log(product(n_z_w[w] + beta)) = sum(np.log(n_z_w[w] + beta))
-        #  lD2 = np.log(product(n_z[d] + V*beta + i -1)) = sum(np.log(n_z[d] + V*beta + i -1))
-        _ld1 = np.log(self.n_documents - 1 + self.n_clusters * self.alpha)
-        _document_size = len(documents)
-        for label in range(0, self.n_clusters, 1):
-            _ln1 = np.log(self.cluster_word_count[label] + self.alpha)
-            _ln2 = 0
-            _ld2 = 0
-            for word in documents:
-                _ln2 += np.log(self.cluster_word_distribution[label].get(word, 0) + self.beta)
-            for j in range(1, _document_size + 1, 1):
-                _ld2 += np.log(self.cluster_word_count[label] + self.vocab_size * self.beta + j - 1)
-            _p[label] = np.exp(_ln1 - _ld1 + _ln2 - _ld2)
-        # normalize the probability vector
-        _normalized_probability_vector = sum(_p)
-        _normalized_probability_vector = _normalized_probability_vector if _normalized_probability_vector > 0 else 1
-        return [probability / _normalized_probability_vector for probability in _p]
+        if self.fast:
+            # Use C++ implementation:
+            return self.model.document_scoring(document,
+                                               self.alpha,
+                                               self.beta,
+                                               self.n_clusters,
+                                               self.vocab_size,
+                                               self.cluster_word_count,
+                                               self.cluster_document_count,
+                                               self.cluster_word_distribution
+                                               )
+        else:
+            # Use Python implementation:
+            _p = [0 for _ in range(0, self.n_clusters, 1)]
+            #  We break the formula into the following pieces
+            #  p = n1 * n2 / (d1 * d2) = np.exp(ln1 - ld1 + ln2 - ld2)
+            #  lN1 = np.log(m_z[z] + alpha)
+            #  lN2 = np.log(D - 1 + K*alpha)
+            #  lN2 = np.log(product(n_z_w[w] + beta)) = sum(np.log(n_z_w[w] + beta))
+            #  lD2 = np.log(product(n_z[d] + V*beta + i -1)) = sum(np.log(n_z[d] + V*beta + i -1))
+            _ld1 = np.log(self.n_documents - 1 + self.n_clusters * self.alpha)
+            _document_size = len(document)
+            for label in range(0, self.n_clusters, 1):
+                _ln1 = np.log(self.cluster_word_count[label] + self.alpha)
+                _ln2 = 0
+                _ld2 = 0
+                for word in document:
+                    _ln2 += np.log(self.cluster_word_distribution[label].get(word, 0) + self.beta)
+                for j in range(1, _document_size + 1, 1):
+                    _ld2 += np.log(self.cluster_word_count[label] + self.vocab_size * self.beta + j - 1)
+                _p[label] = np.exp(_ln1 - _ld1 + _ln2 - _ld2)
+            # normalize the probability vector
+            _normalized_probability_vector = sum(_p)
+            _normalized_probability_vector = _normalized_probability_vector if _normalized_probability_vector > 0 else 1
+            return [probability / _normalized_probability_vector for probability in _p]
 
     def _sampling(self) -> int:
         """
@@ -157,6 +191,41 @@ class GibbsSamplingDirichletMultinomialModeling:
         self.alpha = alpha
         self.beta = beta
 
+    def cluster_similarity(self, cluster_labels: List[int] = None, top_n_words: int = 10, last_n_words: int = 0) -> Dict[str, Dict[str, float]]:
+        """
+        Calculate similarity of learned clusters using top-n-words and last-n-words
+
+        :param cluster_labels: List[int]
+            Cluster labels to delete
+
+        :param top_n_words: int
+            Number of top words to compare
+
+        :param last_n_words: int
+            Number of last words to compare
+
+        :return: Dict[str, Dict[str, float]]
+            Similarity score based on comparison of top-n-words and last-n-words
+        """
+        _cluster_labels: List[int] = [i for i in range(0, self.n_clusters, 1)] if cluster_labels is None else cluster_labels
+        _top_n_words_each_cluster: dict = self.get_top_words_each_cluster(top_n_words=top_n_words if top_n_words > 0 else 10)
+        _last_n_words_each_cluster: dict = self.get_last_words_each_cluster(last_n_words=last_n_words if last_n_words > 0 else 0)
+        _cluster_similarity: Dict[str, Dict[str, float]] = {}
+        for first_cluster in _cluster_labels:
+            _cluster_similarity.update({str(first_cluster): {}})
+            _first_cluster_top_n_words: List[str] = [w[0] for w in _top_n_words_each_cluster.get(str(first_cluster))]
+            for second_cluster in _cluster_labels:
+                if first_cluster == second_cluster:
+                    continue
+                _second_cluster_top_n_words: List[str] = [w[0] for w in _top_n_words_each_cluster.get(str(second_cluster))]
+                _similarity: float = 0
+                for top in _first_cluster_top_n_words:
+                    if top in _second_cluster_top_n_words:
+                        _similarity += 1
+                _similarity = 0 if _similarity == 0 else _similarity / len(_first_cluster_top_n_words)
+                _cluster_similarity[str(first_cluster)].update({str(second_cluster): _similarity})
+        return _cluster_similarity
+
     def delete_cluster(self, cluster_labels: List[int]):
         """
         Delete cluster information
@@ -180,56 +249,61 @@ class GibbsSamplingDirichletMultinomialModeling:
         Fitting GSDMM clustering algorithm
 
         :param documents: List[List[str]]
-            The document token stream
+            Tokenized documents
 
         :return: List[int]
             Cluster labels
         """
         self.n_documents = len(documents)
-        _document_cluster: List[int] = [_ for _ in range(0, self.n_documents, 1)]
-        # initialize the clusters
-        _n_clusters: int = self.n_clusters
-        for i, doc in enumerate(documents):
-            # choose a random initial cluster for the document
-            self.probability_vector = [1.0 / self.n_clusters for _ in range(0, self.n_clusters, 1)]
-            _i: int = self._sampling()
-            _document_cluster[i] = _i
-            self.cluster_document_count[_i] += 1
-            self.cluster_word_count[_i] += len(doc)
-            for word in doc:
-                if word not in self.cluster_word_distribution[_i]:
-                    self.cluster_word_distribution[_i][word] = 0
-                self.cluster_word_distribution[_i][word] += 1
-        for _iter in range(0, self.n_iterations, 1):
-            _total_transfers: int = 0
+        if self.fast:
+            # Use C++ implementation:
+            _document_cluster, self.cluster_word_count, self.cluster_document_count, self.cluster_word_distribution = self.model.fit(documents)
+        else:
+            # Use Python implementation:
+            _document_cluster: List[int] = [_ for _ in range(0, self.n_documents, 1)]
+            # initialize the clusters
+            _n_clusters: int = self.n_clusters
             for i, doc in enumerate(documents):
-                # remove the doc from it's current cluster
-                _old_cluster: int = _document_cluster[i]
-                self.cluster_document_count[_old_cluster] -= 1
-                self.cluster_word_count[_old_cluster] -= len(doc)
+                # choose a random initial cluster for the document
+                self.probability_vector = [1.0 / self.n_clusters for _ in range(0, self.n_clusters, 1)]
+                _i: int = self._sampling()
+                _document_cluster[i] = _i
+                self.cluster_document_count[_i] += 1
+                self.cluster_word_count[_i] += len(doc)
                 for word in doc:
-                    self.cluster_word_distribution[_old_cluster][word] -= 1
-                    # compact dictionary to save space
-                    if self.cluster_word_distribution[_old_cluster][word] == 0:
-                        del self.cluster_word_distribution[_old_cluster][word]
-                # draw sample from distribution to find new cluster
-                self.probability_vector = self._document_scoring(documents=documents[i])
-                _new_cluster: int = self._sampling()
-                # transfer doc to the new cluster
-                if _new_cluster != _old_cluster:
-                    _total_transfers += 1
-                _document_cluster[i] = _new_cluster
-                self.cluster_document_count[_new_cluster] += 1
-                self.cluster_word_count[_new_cluster] += len(doc)
-                for word in doc:
-                    if word not in self.cluster_word_distribution[_new_cluster]:
-                        self.cluster_word_distribution[_new_cluster][word] = 0
-                    self.cluster_word_distribution[_new_cluster][word] += 1
-            _new_cluster_count = sum([1 for v in self.cluster_document_count if v > 0])
-            if _total_transfers == 0 and _new_cluster_count == _n_clusters and _iter > self.n_iterations - 5:
-                break
-            _n_clusters = _new_cluster_count
-        self.n_clusters = _n_clusters
+                    if word not in self.cluster_word_distribution[_i]:
+                        self.cluster_word_distribution[_i][word] = 0
+                    self.cluster_word_distribution[_i][word] += 1
+            for _iter in range(0, self.n_iterations, 1):
+                _total_transfers: int = 0
+                for i, doc in enumerate(documents):
+                    # remove the doc from it's current cluster
+                    _old_cluster: int = _document_cluster[i]
+                    self.cluster_document_count[_old_cluster] -= 1
+                    self.cluster_word_count[_old_cluster] -= len(doc)
+                    for word in doc:
+                        self.cluster_word_distribution[_old_cluster][word] -= 1
+                        # compact dictionary to save space
+                        if self.cluster_word_distribution[_old_cluster][word] == 0:
+                            del self.cluster_word_distribution[_old_cluster][word]
+                    # draw sample from distribution to find new cluster
+                    self.probability_vector = self._document_scoring(document=documents[i])
+                    _new_cluster: int = self._sampling()
+                    # transfer doc to the new cluster
+                    if _new_cluster != _old_cluster:
+                        _total_transfers += 1
+                    _document_cluster[i] = _new_cluster
+                    self.cluster_document_count[_new_cluster] += 1
+                    self.cluster_word_count[_new_cluster] += len(doc)
+                    for word in doc:
+                        if word not in self.cluster_word_distribution[_new_cluster]:
+                            self.cluster_word_distribution[_new_cluster][word] = 0
+                        self.cluster_word_distribution[_new_cluster][word] += 1
+                _new_cluster_count = sum([1 for v in self.cluster_document_count if v > 0])
+                if _total_transfers == 0 and _new_cluster_count == _n_clusters and _iter > self.n_iterations - 5:
+                    break
+                _n_clusters = _new_cluster_count
+            self.n_clusters = _n_clusters
         return _document_cluster
 
     def generate_topic_allocation(self, documents: List[List[str]]) -> List[int]:
@@ -241,19 +315,38 @@ class GibbsSamplingDirichletMultinomialModeling:
         """
         _topic_allocations = []
         for doc in documents:
-            _topic_label = self.predict(documents=doc)
+            _topic_label = self.predict(document=doc)
             _topic_allocations.append(_topic_label)
         return _topic_allocations
 
-    def get_top_words_each_cluster(self, top_n_words: int = 5) -> dict:
+    def get_last_words_each_cluster(self, last_n_words: int = 5) -> Dict[str, Tuple[str, int]]:
         """
-        Get top n words of each cluster
+        Get last-n-words of each cluster
+
+        :param last_n_words: int
+            Number of last words of each cluster
+
+        :return: Dict[str, Tuple[str, int]]
+            Last-n-words and frequency of each cluster
+        """
+        _last_n_words_each_cluster: dict = {}
+        for cluster in range(0, self.n_clusters, 1):
+            _sorted_words_current_cluster = sorted(self.cluster_word_distribution[cluster].items(),
+                                                   key=lambda c: c[1],
+                                                   reverse=True
+                                                   )[last_n_words:]
+            _last_n_words_each_cluster.update({str(cluster): _sorted_words_current_cluster})
+        return _last_n_words_each_cluster
+
+    def get_top_words_each_cluster(self, top_n_words: int = 5) -> Dict[str, Tuple[str, int]]:
+        """
+        Get top-n-words of each cluster
 
         :param top_n_words: int
-            Number of top n words of each cluster
+            Number of top words of each cluster
 
-        :return: dict
-            Top n words of each cluster
+        :return: Dict[str, Tuple[str, int]]
+            Top-n-words and frequency of each cluster
         """
         _top_n_words_each_cluster: dict = {}
         for cluster in range(0, self.n_clusters, 1):
@@ -264,29 +357,121 @@ class GibbsSamplingDirichletMultinomialModeling:
             _top_n_words_each_cluster.update({str(cluster): _sorted_words_current_cluster})
         return _top_n_words_each_cluster
 
-    def predict(self, documents: List[str]) -> int:
+    def merge_cluster(self, cluster_labels: List[int]):
+        """
+        Merge clusters
+
+        :param cluster_labels: List[int]
+            Cluster labels to merge (first cluster label will be used as base)
+        """
+        for i, cluster in enumerate(cluster_labels):
+            if i > 0:
+                self.n_clusters -= 1
+                self.cluster_word_count[cluster_labels[0]] += self.cluster_word_count[cluster]
+                self.cluster_document_count[cluster_labels[0]] += self.cluster_document_count[cluster]
+                for word, freq in self.cluster_word_distribution[cluster].items():
+                    if word in self.cluster_word_distribution[cluster_labels[0]].keys():
+                        self.cluster_word_distribution[cluster_labels[0]][word] += freq
+                    else:
+                        self.cluster_word_distribution[cluster_labels[0]].update({word: freq})
+                del self.cluster_word_count[cluster]
+                del self.cluster_word_distribution[cluster]
+
+    def predict(self, document: List[str]) -> int:
         """
         Predict cluster label
 
-        :param documents: List[str]
-            The document token stream
+        :param document: List[str]
+            Tokenized document
 
         :return: int
             Cluster label
         """
-        return np.array(self._document_scoring(documents=documents)).argmax()
+        return np.array(self._document_scoring(document=document)).argmax()
 
-    def predict_proba(self, documents: List[str]) -> List[float]:
+    def predict_proba(self, document: List[str]) -> List[float]:
         """
         Predict probabilities for each document
 
-        :param documents: List[str]
-            The document token stream
+        :param document: List[str]
+            Tokenized document
 
         :return: List[np.array]
             Probability vector for each document
         """
-        return self._document_scoring(documents=documents)
+        return self._document_scoring(document=document)
+
+    def prepare_visualization(self, documents: List[List[str]]) -> pyLDAvis:
+        """
+        Prepare documents for visualization from trained model
+
+        :param documents: List[List[str]]
+            Tokenized documents
+
+        :return: pyLDAvis
+            Prepared word matrix, documents distances, vocabulary and word counts using pyLDAvis library
+        """
+        _voc: List[str] = []
+        for cluster in self.cluster_word_distribution:
+            _voc.extend(list(cluster.keys()))
+        _vocabulary: List[str] = list(set(_voc))
+        _doc_topic_distances: List[List[float]] = [self.predict_proba(doc) for doc in documents]
+        for doc in _doc_topic_distances:
+            for word in doc:
+                assert not isinstance(word, complex)
+        _doc_len = [len(doc) for doc in documents]
+        _word_counts_map: dict = {}
+        for doc in documents:
+            for word in doc:
+                _word_counts_map[word] = _word_counts_map.get(word, 0) + 1
+        _word_counts: list = [_word_counts_map[term] for term in _vocabulary]
+        _doc_topic_distances_ext: list = [[v if not math.isnan(v) else 1 / self.n_clusters for v in d] for d in _doc_topic_distances]
+        _doc_topic_distances_ext = [d if sum(d) > 0 else [1 / self.n_clusters] * self.n_clusters for d in _doc_topic_distances_ext]
+        for doc in _doc_topic_distances_ext:
+            for f in doc:
+                assert not isinstance(f, complex)
+        assert (pd.DataFrame(_doc_topic_distances_ext).sum(axis=1) < 0.999).sum() == 0
+        _word_matrix: list = []
+        for cluster in self.cluster_word_distribution:
+            _total: float = sum([frequency for word, frequency in cluster.items()])
+            assert not math.isnan(_total)
+            if _total == 0:
+                _row: list = [(1 / len(_vocabulary))] * len(_vocabulary)
+            else:
+                _row: list = [cluster.get(word, 0) / _total for word in _vocabulary]
+            for word in _row:
+                assert not isinstance(word, complex)
+            _word_matrix.append(_row)
+        return pyLDAvis.prepare(topic_term_dists=_word_matrix,
+                                doc_topic_dists=_doc_topic_distances_ext,
+                                doc_lengths=_doc_len,
+                                vocab=_vocabulary,
+                                term_frequency=_word_counts,
+                                R=30,
+                                lambda_step=0.01,
+                                sort_topics=False
+                                )
+
+    @staticmethod
+    def save_visualization(file_path: str, vis_data, **kwargs):
+        """
+        Save topic visualization as interactive html file
+
+        :param file_path: str
+            Complete file path
+
+        :param vis_data: pyLDAvis
+            Prepared data using pyLDAvis library
+
+        :param kwargs: dict
+            Key-word arguments for handling cloud information
+        """
+        DataExporter(obj=vis_data,
+                     file_path=file_path,
+                     bucket_name=kwargs.get('bucket'),
+                     region=kwargs.get('region'),
+                     **dict(topic_clustering=True)
+                     ).file()
 
     def word_importance_each_cluster(self) -> List[dict]:
         """
